/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ELFSIGNRANGE_DOT_H
#define	__ELFSIGNRANGE_DOT_H

#include <sys/types.h>
#include <sys/machelf.h>
#include <sys/elfsignrange.h>

#ifdef	__cplusplus
extern "C" {
#endif

/* Define _ELF64 and non_ELF6 mappings for routine that follows */
#if	defined(_ELF64)
#define	_elf_sign_range		_elf64_sign_range
#else
#define	_elf_sign_range		_elf32_sign_range
#endif

/* Machine and class specific ELF interfaces - available to libelf and krtld */
extern	int	_elf_sign_range(elf_sign_range_cb_func, Ehdr *,
		    Phdr *, Shdr *, size_t, Elf_Signrange, void *);

/*
 * Provide a simple summing implementation.  This provides a means of generating
 * a DT_CHECKSUM, and is probably of little interest to anyone else.  However,
 * for now this implementation is designed to be available for both kernel and
 * userland use, as a simple means of testing a checksum/signature framework
 * between the two.
 */
#define	_ELF_MSW(l)	(((l) >> 16) & 0x0000ffffL)
#define	_ELF_LSW(l)	((l) & 0x0000ffffL)

/* Checksum data structure supplied as cookie to checksum routines */
typedef struct {
	char	*ecs_base;
	long	ecs_sum;
	uint_t	ecs_flags;
} _Elf_Checksum_desc_t;

/* elf_sign_range_cb_func() callback */
extern void	_elf_checksum_callback(size_t, size_t, void *);
/* Checksum cookie finalization */
extern void	_elf_checksum_fini(void *);
/* Checksum cookie initialization - supplied object base address and flags */
extern void	_elf_checksum_init(void *, char *, uint_t);

#ifdef	__cplusplus
}
#endif

#endif /* __ELFSIGNRANGE_DOT_H */
