/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <grub/command.h>
#include <grub/device.h>
#include <grub/disk.h>
#include <grub/dl.h>
#include <grub/misc.h>
#include <grub/term.h>

#include "zvcons.h"

/* No dynamic loading in zvboot. */
int
grub_dl_ref(grub_dl_t mod)
{
	return (1);
}

int
grub_dl_unref(grub_dl_t mod)
{
	return (1);
}

/* Miscellaneous. */
void
grub_exit(void)
{
	extern void panic(char *fmt, ...);

	panic("grub_exit() called.");
	for (;;)
		;
}

/*
 * Set to 'zvboot' for zvboot specific details, set to 'all' for
 * more.
 */
static char *grub_debug_value = "";

const char *
grub_env_get(const char *var)
{
	if (grub_strcmp(var, "debug") == 0)
		return (grub_debug_value);

	return (NULL);
}

grub_err_t
grub_env_set(const char *var, const char *val)
{
	if (grub_strcmp(var, "debug") == 0) {
		if (val[0] != '\0') {
			if (grub_debug_value[0] != '\0')
				grub_free(grub_debug_value);
			grub_debug_value = grub_strdup(val);
		}
	}

	return (GRUB_ERR_NONE);
}

grub_command_t
grub_register_command_prio(const char *name, grub_command_func_t func,
    const char *summary, const char *description, int prio)
{
	return (NULL);
}

void
grub_unregister_command(grub_command_t cmd)
{
}

grub_uint64_t
grub_get_time_ms(void)
{
	/*
	 * Nothing needs real time, so we provide a simple
	 * incrementing counter just in case some code compares the
	 * results.
	 */
	static grub_uint64_t t;

	return (t++);
}

/* Terminal support. */
void (*grub_xputs) (const char *str) = zcons_puts;
struct grub_term_input *grub_term_inputs = NULL;

int
grub_getkey(void)
{
	return (GRUB_TERM_NO_KEY);
}

void
grub_refresh(void)
{
}
