/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#if defined(__lint)

#include "zv_asm.h"

int silence_lint_warnings = 0;

uint64_t
getcr3(void)
{ return (0); }

/*ARGSUSED*/
uint64_t
do_hypercall(uint64_t, uint64_t, ...)
{ return (0); }

#else /* __lint */

#include <sys/asm_linkage.h>

	.text
	.globl _start
_start:
	/*
	 * At entry we are in 64 bit paging mode with interrupts disabled.
	 *
	 * %RDI has a pointer to our zv_start_data
	 * %RSP has lots of room and is 16 byte aligned.
	 *
	 * really not much more to do here so just
	 * go off into C to handle everything.
	 * We never actually return here either, but
	 * just call into the loaded kernel.
	 */
	jmp	boot

	ENTRY(getcr3)
	movq	%cr3, %rax
	ret
	SET_SIZE(getcr3)

	ENTRY(do_hypercall)
        pushq   %rbx
        pushq   %rcx
        pushq   %rdx

        xorq    %rax, %rax
        cpuid
        cmpl    $0x68747541, %ebx       /* "Auth" part of AuthenticAMD */
        jne     1f
        leaq    amd_vmmcall_insn(%rip), %rax
        jmp     2f
1:
        leaq    intel_vmcall_insn(%rip), %rax
2:
        popq    %rdx
        popq    %rcx
        popq    %rbx
        jmp     *%rax

intel_vmcall_insn:
        vmcall
        ret

amd_vmmcall_insn:
        vmmcall
        ret

	SET_SIZE(do_hypercall)

#endif /* __lint */
