/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef	_ZVMEM_H
#define	_ZVMEM_H

#ifdef	__cplusplus
extern "C" {
#endif

#ifndef NULL
#define	NULL (void *)0
#endif

#define KBYTE(x) ((x) * 1024ul)
#define MBYTE(x) (KBYTE(x) * 1024ul)
#define GBYTE(x) (MBYTE(x) * 1024ul)

#define ROUND_UP(val, align) (((val) + ((align)-1)) & ~(uint64_t)((align)-1))

#if defined(amd64)

#define	PGSIZE	KBYTE(4)

/*
 * page table bit defines
 */
#define	PT_VALID	(0x001) /* a valid translation is present */
#define	PT_WRITABLE	(0x002) /* the page is writable */
#define	PT_USER		(0x004) /* the page is accessible by user mode */
#define	PT_WRITETHRU	(0x008) /* write back caching is disabled (non-PAT) */
#define	PT_NOCACHE	(0x010) /* page is not cacheable (non-PAT) */
#define	PT_REF		(0x020) /* page was referenced */
#define	PT_MOD		(0x040) /* page was modified */
#define	PT_PAGESIZE	(0x080) /* above level 0, indicates a large page */
#define	PT_GLOBAL	(0x100) /* the mapping is global */
#define PT_NOCONSIST	(0x400) /* PTE has no mapping list entry */
typedef uint64_t x86pte_t;

extern x86pte_t *find_pte(uintptr_t va, uint_t level);
extern void map_pa_at_va(uint64_t pa, void *va, uint_t level);

#endif /* amd64 */

#define PGOFFSET	(PGSIZE - 1)
#define PGMASK		~PGOFFSET

extern uintptr_t last_phys;

extern void *page_alloc(size_t size);
extern void *memmove(void *s, const void *s0, size_t n);

#ifdef	__cplusplus
}
#endif

#endif	/* _ZVMEM_H */
