/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2011, 2012, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <stdarg.h>
#include <sys/zvmm/zvmm_hcall.h>
#include <sys/zvmm/zvterm_common.h>

#include "zv_asm.h"
#include "zv_mem.h"
#include "zvcons.h"

#include <grub/misc.h>

#define	EAGAIN	(11)

/*
 * Primitive version of panic, prints a message then resets the system.
 */
void
panic(char *fmt, ...)
{
	va_list	args;

	grub_printf("\nPANIC: zvboot: ");
	va_start(args, fmt);
	(void) grub_vprintf(fmt, args);
	va_end(args);
	grub_printf("\n");

	do_hypercall(ZVMM_CNTRL_EXTID, ZVCNTRL_PAUSE);
	do_hypercall(ZVMM_CNTRL_EXTID, ZVCNTRL_HALT);
}

void
zcons_puts(const char *s)
{
	while (*s != 0) {
		zcons_putchar(*s);
		++s;
	}
}

void
zcons_putchar(char c)
{
	int checks = 10000;

	while (checks--) {
		if (do_hypercall(ZVMM_CONS_EXTID, ZVTERM_WRITE, 0, 0) != EAGAIN)
			break;

		(void) do_hypercall(ZVMM_CNTRL_EXTID, ZVCNTRL_PAUSE);
	}

	(void) do_hypercall(ZVMM_CONS_EXTID, ZVTERM_PUTCHAR, c);
}

char
zcons_getchar(void)
{
	for (;;) {
		if (do_hypercall(ZVMM_CONS_EXTID, ZVTERM_READ, 0, 0) != EAGAIN)
			break;

		(void) do_hypercall(ZVMM_CNTRL_EXTID, ZVCNTRL_PAUSE);
	}

	return (do_hypercall(ZVMM_CONS_EXTID, ZVTERM_GETCHAR));
}

int
zcons_ischar(void)
{
	return (do_hypercall(ZVMM_CONS_EXTID, ZVTERM_READ, 0, 0) == 0);
}
