/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (C) 2009, Free Software Foundation, Inc.
 *  Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Emulate required GPG error enums for use by the MPI library in GRUB.
 * This for Solaris Verified boot's use of libgcrypt functions in multiboot.
 */

#ifndef _GPG_ERROR_H
#define	_GPG_ERROR_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * libgcrypt error values and functions as adapted for GRUB.
 */

typedef enum {
	GPG_ERR_NO_ERROR = 0,
	GPG_ERR_INV_ARG = 45,
	GPG_ERR_INTERNAL = 63,
	GPG_ERR_INV_OBJ = 65,
	GPG_ERR_TOO_SHORT = 66,
	GPG_ERR_TOO_LARGE = 67,
	GPG_ERR_MISSING_ERRNO = 16381,
	GPG_ERR_UNKNOWN_ERRNO = 16382,
	/* Error codes after this are for mapping grub_errno values: */
#define	GPG_ERR_SYSTEM_ERROR	(1 << 15)
	GPG_ERR_CODE_DIM = 65536	/* Highest error code + 1 */
} gpg_err_code_t;

typedef unsigned int gpg_error_t;
/* Wrappers for the libgpg-error library. */
typedef gpg_error_t gcry_error_t;
typedef gpg_err_code_t gcry_err_code_t;

gpg_err_code_t gpg_err_code_from_syserror(void);

static inline gpg_error_t
gcry_error(gcry_err_code_t code) {
	return (code);
}

static inline gpg_error_t
gpg_error_from_syserror(void) {
	return (gpg_err_code_from_syserror());
}

#ifdef	__cplusplus
}
#endif

#endif	/* _GPG_ERROR_H */
