/* wanboot.h - EFI-based WANBoot */
/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 *
 *  GRUB is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  GRUB is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GRUB.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GRUB_EFI_WANBOOT_HEADER
#define GRUB_EFI_WANBOOT_HEADER 1

#define WANBOOT_GUID   \
  { 0x1fbe91b6, 0x1110, 0x4bca, \
    { 0x8b, 0xaa, 0xd6, 0x4d, 0x68, 0x88, 0x31, 0x37 } \
  }

/*
 * UEFI WANBoot variable names
 */
#define WB_BI_CID               "client-id"
#define WB_BI_MACADDR           "macaddr"
#define WB_BI_HOST_IP           "volatile-host-ip"
#define WB_BI_SUBNET_MASK       "volatile-subnet-mask"
#define WB_BI_ROUTER_IP         "volatile-router-ip"
#define WB_BI_SERVER_IP         "volatile-server-ip"
#define WB_BI_SERVER_PORT       "volatile-server-port"
#define WB_BI_SERVER_RSRC       "volatile-server-resource"
#define	WB_BI_BOOTP_RESPONSE	"bootp-response"
#define WB_BI_USE_CLIENT_KEYS	"use-client-keys"
#define WB_BI_HMAC_KEY          "hmac-sha256"
#define WB_BI_AES_KEY           "aes"

/*
 * WANBoot variable default values
 */
#define WB_DEF_SERVER_PORT      5555
#define WB_DEF_SERVER_RSRC      "/cgi-bin/wanboot-cgi"

/*
 * UEFI variable lengths as specified
 */
#define WB_CLIENT_ID_LEN        7	/* client-id may be longer, but */
					/* 14 hex digits are used by the AI */
					/* Server, so limiting it to 7 chars */
#define WB_CLIENT_ID_STR_LEN    (WB_CLIENT_ID_LEN*2)
#define	WB_MACADDR_LEN		6	/* MAC address length */
#define	WB_SERVER_RSRC_LEN	80	/* server resource length */
#define	WB_NONCE_LEN		20	/* nonce len in octets */
#define WB_NONCE_STR_LEN	(WB_NONCE_LEN * 2)
					/* Like client id, nonce is used */
					/* here is a string of uppercase hex */
					/* digits */
#define	WB_IPADDR_STR_LEN	15	/* IP address string length */

/*
 * Crypto byte lengths
 */
#define	SHA256_DIGEST_LEN	32	/* SHA-256 digest */
#define	AES128_KEY_LEN		16	/* 128-bit AES key */
#define	AES192_KEY_LEN		24	/* 192-bit AES key */
#define	AES256_KEY_LEN		32	/* 256-bit AES key */

/*
 * GRUB env variable for indicate WANBoot is being done
 */
#define	WANBOOT_ENV_ENABLED	"wanboot_enabled"

/*
 * GRUB env variables for https parameters
 */
#define	HTTPS_ENV_CERTSTORE	"https_certstore"
#define	HTTPS_ENV_KEYSTORE	"https_keystore"
#define	HTTPS_ENV_TRUSTSTORE	"https_truststore"
#define	HTTPS_ENV_CLIENTAUTH	"https_client_auth"
#define	HTTPS_ENV_SERVERAUTH	"https_server_auth"

/*
 * Files on WANBoot filesystem for https key/certificates
 */
#define	WBFS_FILE_CERTSTORE	"(wbfs)/certstore"
#define	WBFS_FILE_KEYSTORE	"(wbfs)/keystore"
#define	WBFS_FILE_TRUSTSTORE	"(wbfs)/truststore"

#endif /* ! GRUB_EFI_WANBOOT_HEADER */
