#!/usr/bin/python

#
# Copyright (c) 2012, 2021, Oracle and/or its affiliates.
#

"""
C functions from /usr/lib/libima.so
"""

import ctypes as C

from libima.cstruct import IMA_INITIATOR_AUTHPARMS, IMA_OID, IMA_OID_LIST

_LIBIMA = C.CDLL("libima.so.1")

# aliases from libima.h
IMA_UINT = C.c_int
IMA_STATUS = IMA_UINT

# function mapping.  Each line is:  (function name, return type, args)
_funcs = [
    ("IMA_FreeMemory", IMA_STATUS, (C.c_void_p,)),
    ("IMA_GetLhbaOidList", IMA_STATUS, (C.POINTER(C.POINTER(IMA_OID_LIST)),)),
    ("IMA_GetInitiatorAuthParms", IMA_STATUS, (
        IMA_OID, IMA_UINT, C.POINTER(IMA_INITIATOR_AUTHPARMS))),
    ("IMA_SetInitiatorAuthParms", IMA_STATUS, (
        IMA_OID, IMA_UINT, C.POINTER(IMA_INITIATOR_AUTHPARMS)))
]
v = vars()
for (function, restype, args) in _funcs:
    v[function] = getattr(_LIBIMA, function)
    v[function].restype = restype
    v[function].argtypes = args
