#
# Copyright (c) 2009, 2021, Oracle and/or its affiliates.
#

LIBRARY=	libntfs.a
VERS=		.10

#
# All relative to SRCDIR
#

OBJECTS=	attrib.o \
                attrlist.o \
                bitmap.o \
                bootsect.o \
                collate.o \
                compress.o \
                crypto.o \
                device.o \
                device_io.o \
                dir.o \
                index.o \
                inode.o \
                lcnalloc.o \
                logfile.o \
                logging.o \
                mft.o \
                misc.o \
                mst.o \
                runlist.o \
                security.o \
                unistr.o \
                version.o \
                volume.o

# include library definitions
include	$(SRC)/lib/Makefile.lib
include $(SRC)/lib/Makefile.lib.64

SRCDIR =	../common/libntfs
MAPFILES =	../common/mapfile-vers

# The following objects are unused.  Refer to comments in Makefile.master.
UNUSEDOBJS =	compat.o \
		debug.o
UNUSEDFILES =	$(UNUSEDOBJS:%.o=$(SRCDIR)/%.c)
POST_PROCESS_SO += $(ACCESS_UNUSEDFILES)

CERRWARN +=	-erroff=E_ENUM_VAL_OVERFLOWS_INT_MAX
CERRWARN +=	-erroff=E_STRUCT_DERIVED_FROM_FLEX_MBR
CERRWARN +=	-erroff=E_END_OF_LOOP_CODE_NOT_REACHED
CERRWARN +=	-erroff=E_LOOP_NOT_ENTERED_AT_TOP

LIBS =		$(DYNLIB)

CPPFLAGS +=	-DHAVE_CONFIG_H \
		-DLTVERSION_LIBNTFS=\"10:0:0\" \
		-I../common/include/ntfs
DYNFLAGS +=	$(ZINTERPOSE)
LDLIBS +=	-lc

STRICTLINTFLAGS64 += -m -v -u $(LDFLAGS64)
STRICT_ALWAYS_LINT_DEFS += -erroff=E_CONSTANT_CONDITION
STRICT_ALWAYS_LINT_DEFS += -erroff=E_STATIC_UNUSED
STRICT_ALWAYS_LINT_DEFS += -erroff=E_FUNC_RET_ALWAYS_IGNOR2
STRICT_ALWAYS_LINT_DEFS += -erroff=E_FUNC_RET_MAYBE_IGNORED2
STRICT_ALWAYS_LINT_DEFS += -erroff=E_CONST_EXPR
STRICT_ALWAYS_LINT_DEFS += -erroff=E_EQUALITY_NOT_ASSIGNMENT
STRICT_ALWAYS_LINT_DEFS += -erroff=E_ENUM_VAL_OVERFLOWS_INT_MAX
STRICT_ALWAYS_LINT_DEFS += -erroff=E_BAD_PTR_CAST_ALIGN
STRICT_ALWAYS_LINT_DEFS += -erroff=E_STRUCT_DERIVED_FROM_FLEX_MBR
STRICT_ALWAYS_LINT_DEFS += -erroff=E_FUNC_SET_NOT_USED

#
# Studio 12.6 defines __BYTE_ORDER__ but not the other macros expected by
# ntfs/endians.h, leading to an #error.
# Unsetting __BYTE_ORDER__ restores the same logic as used under Studio 12.4.
# GCC 5+ also sees this issue.
#
CPPFLAGS += -U__BYTE_ORDER__

#
# The use of NTFS_BUG() to insert SEGV-inducing code using a NULL ptr deref
# means that error type needs suppressing here. Because it is defined in a
# header file, a #pragma won't work, and we must bash the entirity of both
# files with the erroff hammer.
#
$(BUILDDIR)/pics/lcnalloc.o $(BUILDDIR)/pics/mft.o := \
	CERRWARN += -erroff=SEC_NULL_PTR_DEREF

.KEEP_STATE_FILE:	$(BUILDDIR)/.make.state.$(MACH)

all install: stub $(BUILDLIBS)

install: stubinstall all $(ROOTLIBS64) $(ROOTLINKS64)

stubinstall: stub $(STUBROOTLIBS64) $(STUBROOTLINKS64)

lint: strictlintcheck64

include $(SRC)/lib/Makefile.targ
