/*
 * Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
 */
/*
 * Description
 * mpapi-sun.c - Implements the Sun Extension to the Multipath Management
 * API Version 1.0
 */

#include <dlfcn.h>
#include <pthread.h>
#include "mpapi.h"
#include "mpapi-sun.h"
#include "mpapi-plugin.h"

extern MPPLUGININFO_T plugintable[MP_MAX_NUM_PLUGINS];

extern pthread_mutex_t mp_lib_mutex;
extern MP_STATUS validate_object(MP_OID obj, MP_OBJECT_TYPE objType,
		MP_UINT32 flag);

MP_STATUS
Sun_MP_SendScsiCmd(
	MP_OID pathOid, struct uscsi_cmd *cmd)
{
	Sun_MP_SendScsiCmdFn PassFunc;
	MP_UINT32 index;
	MP_STATUS status;

	if ((status = validate_object(pathOid, MP_OBJECT_TYPE_PATH_LU,
	    MP_OBJECT_TYPE_MATCH)) != MP_STATUS_SUCCESS) {
		return (status);
	}

	(void) pthread_mutex_lock(&mp_lib_mutex);

	index = pathOid.ownerId - 1;
	if (plugintable[index].hdlPlugin != NULL) {
		PassFunc = (Sun_MP_SendScsiCmdFn)
		    dlsym(plugintable[index].hdlPlugin,
		    "Sun_MP_SendScsiCmd");

		if (PassFunc != NULL) {
			status = PassFunc(pathOid, cmd);
		} else {
			status = MP_STATUS_UNSUPPORTED;
		}
	} else {
		status = MP_STATUS_FAILED;
	}

	(void) pthread_mutex_unlock(&mp_lib_mutex);
	return (status);
}

MP_STATUS
Sun_MP_GetLogicalUnitOid(
	MP_OID pluginOid, MP_CHAR *deviceFileName, MP_OID *oid)
{
	MP_UINT32 index;
	MP_STATUS status;
	MP_UINT64 osn;

	if ((status = validate_object(pluginOid, MP_OBJECT_TYPE_PLUGIN,
	    MP_OBJECT_TYPE_MATCH)) != MP_STATUS_SUCCESS) {
		return (status);
	}

	(void) pthread_mutex_lock(&mp_lib_mutex);

	index = pluginOid.ownerId - 1;
	if (plugintable[index].hdlPlugin != NULL) {
		if (pluginOid.objectType == MP_OBJECT_TYPE_PLUGIN) {
			Sun_MP_GetLogicalUnitOidFn PassFunc;
			PassFunc = (Sun_MP_GetLogicalUnitOidFn)
			    dlsym(plugintable[index].hdlPlugin,
			    "Sun_MP_GetLogicalUnitOid");

			if (PassFunc != NULL) {
				status = PassFunc(deviceFileName, &osn);
			} else {
				status = MP_STATUS_UNSUPPORTED;
			}
		} else {
			status = MP_STATUS_INVALID_PARAMETER;
		}
	} else {
		status = MP_STATUS_INVALID_PARAMETER;
	}

	if (status == MP_STATUS_SUCCESS) {
		oid->objectType = MP_OBJECT_TYPE_MULTIPATH_LU;
		oid->ownerId = pluginOid.ownerId;
		oid->objectSequenceNumber = osn;
	}

	(void) pthread_mutex_unlock(&mp_lib_mutex);
	return (status);
}
