/*
 * Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 */

/*
 * This program is copyright Alec Muffett 1993. The author disclaims all
 * responsibility or liability with respect to it's usage or its effect
 * upon hardware or computer systems, and maintains copyright as set out
 * in the "LICENCE" document which accompanies distributions of Crack v4.0
 * and upwards.
 */

#include "packer.h"
#include <search.h>

typedef struct stored_password {
	int		type;	/* DICTIONARY_WORD, REVERSE_DICTIONARY_WORD */
	PWDICT		*pwp;		/* password dictionary */
	uint32_t	pwd_notfound;	/* password not found status */
	char		password[1];	/* the password itself */
} stored_password_t;

static const char *r_destructors[] = {
	":",			/* noop - must do this to test raw word. */

	"/?p@?p",		/* purging out punctuation/symbols/junk */
	"/?s@?s",
	"/?X@?X",
	/* attempt reverse engineering of password strings */
	"/$s$s",
	"/$s$s/0s0o",
	"/$s$s/0s0o/2s2a",
	"/$s$s/0s0o/2s2a/3s3e",
	"/$s$s/0s0o/2s2a/3s3e/5s5s",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1i",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1l",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1i/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1i/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1l/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/1s1l/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/5s5s/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/1s1i",
	"/$s$s/0s0o/2s2a/3s3e/1s1l",
	"/$s$s/0s0o/2s2a/3s3e/1s1i/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/1s1i/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/1s1l/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/1s1l/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/4s4h",
	"/$s$s/0s0o/2s2a/3s3e/4s4a",
	"/$s$s/0s0o/2s2a/3s3e/4s4h",
	"/$s$s/0s0o/2s2a/5s5s",
	"/$s$s/0s0o/2s2a/5s5s/1s1i",
	"/$s$s/0s0o/2s2a/5s5s/1s1l",
	"/$s$s/0s0o/2s2a/5s5s/1s1i/4s4a",
	"/$s$s/0s0o/2s2a/5s5s/1s1i/4s4h",
	"/$s$s/0s0o/2s2a/5s5s/1s1l/4s4a",
	"/$s$s/0s0o/2s2a/5s5s/1s1l/4s4h",
	"/$s$s/0s0o/2s2a/5s5s/4s4a",
	"/$s$s/0s0o/2s2a/5s5s/4s4h",
	"/$s$s/0s0o/2s2a/5s5s/4s4a",
	"/$s$s/0s0o/2s2a/5s5s/4s4h",
	"/$s$s/0s0o/2s2a/1s1i",
	"/$s$s/0s0o/2s2a/1s1l",
	"/$s$s/0s0o/2s2a/1s1i/4s4a",
	"/$s$s/0s0o/2s2a/1s1i/4s4h",
	"/$s$s/0s0o/2s2a/1s1l/4s4a",
	"/$s$s/0s0o/2s2a/1s1l/4s4h",
	"/$s$s/0s0o/2s2a/4s4a",
	"/$s$s/0s0o/2s2a/4s4h",
	"/$s$s/0s0o/2s2a/4s4a",
	"/$s$s/0s0o/2s2a/4s4h",
	"/$s$s/0s0o/3s3e",
	"/$s$s/0s0o/3s3e/5s5s",
	"/$s$s/0s0o/3s3e/5s5s/1s1i",
	"/$s$s/0s0o/3s3e/5s5s/1s1l",
	"/$s$s/0s0o/3s3e/5s5s/1s1i/4s4a",
	"/$s$s/0s0o/3s3e/5s5s/1s1i/4s4h",
	"/$s$s/0s0o/3s3e/5s5s/1s1l/4s4a",
	"/$s$s/0s0o/3s3e/5s5s/1s1l/4s4h",
	"/$s$s/0s0o/3s3e/5s5s/4s4a",
	"/$s$s/0s0o/3s3e/5s5s/4s4h",
	"/$s$s/0s0o/3s3e/5s5s/4s4a",
	"/$s$s/0s0o/3s3e/5s5s/4s4h",
	"/$s$s/0s0o/3s3e/1s1i",
	"/$s$s/0s0o/3s3e/1s1l",
	"/$s$s/0s0o/3s3e/1s1i/4s4a",
	"/$s$s/0s0o/3s3e/1s1i/4s4h",
	"/$s$s/0s0o/3s3e/1s1l/4s4a",
	"/$s$s/0s0o/3s3e/1s1l/4s4h",
	"/$s$s/0s0o/3s3e/4s4a",
	"/$s$s/0s0o/3s3e/4s4h",
	"/$s$s/0s0o/3s3e/4s4a",
	"/$s$s/0s0o/3s3e/4s4h",
	"/$s$s/0s0o/5s5s",
	"/$s$s/0s0o/5s5s/1s1i",
	"/$s$s/0s0o/5s5s/1s1l",
	"/$s$s/0s0o/5s5s/1s1i/4s4a",
	"/$s$s/0s0o/5s5s/1s1i/4s4h",
	"/$s$s/0s0o/5s5s/1s1l/4s4a",
	"/$s$s/0s0o/5s5s/1s1l/4s4h",
	"/$s$s/0s0o/5s5s/4s4a",
	"/$s$s/0s0o/5s5s/4s4h",
	"/$s$s/0s0o/5s5s/4s4a",
	"/$s$s/0s0o/5s5s/4s4h",
	"/$s$s/0s0o/1s1i",
	"/$s$s/0s0o/1s1l",
	"/$s$s/0s0o/1s1i/4s4a",
	"/$s$s/0s0o/1s1i/4s4h",
	"/$s$s/0s0o/1s1l/4s4a",
	"/$s$s/0s0o/1s1l/4s4h",
	"/$s$s/0s0o/4s4a",
	"/$s$s/0s0o/4s4h",
	"/$s$s/0s0o/4s4a",
	"/$s$s/0s0o/4s4h",
	"/$s$s/2s2a",
	"/$s$s/2s2a/3s3e",
	"/$s$s/2s2a/3s3e/5s5s",
	"/$s$s/2s2a/3s3e/5s5s/1s1i",
	"/$s$s/2s2a/3s3e/5s5s/1s1l",
	"/$s$s/2s2a/3s3e/5s5s/1s1i/4s4a",
	"/$s$s/2s2a/3s3e/5s5s/1s1i/4s4h",
	"/$s$s/2s2a/3s3e/5s5s/1s1l/4s4a",
	"/$s$s/2s2a/3s3e/5s5s/1s1l/4s4h",
	"/$s$s/2s2a/3s3e/5s5s/4s4a",
	"/$s$s/2s2a/3s3e/5s5s/4s4h",
	"/$s$s/2s2a/3s3e/5s5s/4s4a",
	"/$s$s/2s2a/3s3e/5s5s/4s4h",
	"/$s$s/2s2a/3s3e/1s1i",
	"/$s$s/2s2a/3s3e/1s1l",
	"/$s$s/2s2a/3s3e/1s1i/4s4a",
	"/$s$s/2s2a/3s3e/1s1i/4s4h",
	"/$s$s/2s2a/3s3e/1s1l/4s4a",
	"/$s$s/2s2a/3s3e/1s1l/4s4h",
	"/$s$s/2s2a/3s3e/4s4a",
	"/$s$s/2s2a/3s3e/4s4h",
	"/$s$s/2s2a/3s3e/4s4a",
	"/$s$s/2s2a/3s3e/4s4h",
	"/$s$s/2s2a/5s5s",
	"/$s$s/2s2a/5s5s/1s1i",
	"/$s$s/2s2a/5s5s/1s1l",
	"/$s$s/2s2a/5s5s/1s1i/4s4a",
	"/$s$s/2s2a/5s5s/1s1i/4s4h",
	"/$s$s/2s2a/5s5s/1s1l/4s4a",
	"/$s$s/2s2a/5s5s/1s1l/4s4h",
	"/$s$s/2s2a/5s5s/4s4a",
	"/$s$s/2s2a/5s5s/4s4h",
	"/$s$s/2s2a/5s5s/4s4a",
	"/$s$s/2s2a/5s5s/4s4h",
	"/$s$s/2s2a/1s1i",
	"/$s$s/2s2a/1s1l",
	"/$s$s/2s2a/1s1i/4s4a",
	"/$s$s/2s2a/1s1i/4s4h",
	"/$s$s/2s2a/1s1l/4s4a",
	"/$s$s/2s2a/1s1l/4s4h",
	"/$s$s/2s2a/4s4a",
	"/$s$s/2s2a/4s4h",
	"/$s$s/2s2a/4s4a",
	"/$s$s/2s2a/4s4h",
	"/$s$s/3s3e",
	"/$s$s/3s3e/5s5s",
	"/$s$s/3s3e/5s5s/1s1i",
	"/$s$s/3s3e/5s5s/1s1l",
	"/$s$s/3s3e/5s5s/1s1i/4s4a",
	"/$s$s/3s3e/5s5s/1s1i/4s4h",
	"/$s$s/3s3e/5s5s/1s1l/4s4a",
	"/$s$s/3s3e/5s5s/1s1l/4s4h",
	"/$s$s/3s3e/5s5s/4s4a",
	"/$s$s/3s3e/5s5s/4s4h",
	"/$s$s/3s3e/5s5s/4s4a",
	"/$s$s/3s3e/5s5s/4s4h",
	"/$s$s/3s3e/1s1i",
	"/$s$s/3s3e/1s1l",
	"/$s$s/3s3e/1s1i/4s4a",
	"/$s$s/3s3e/1s1i/4s4h",
	"/$s$s/3s3e/1s1l/4s4a",
	"/$s$s/3s3e/1s1l/4s4h",
	"/$s$s/3s3e/4s4a",
	"/$s$s/3s3e/4s4h",
	"/$s$s/3s3e/4s4a",
	"/$s$s/3s3e/4s4h",
	"/$s$s/5s5s",
	"/$s$s/5s5s/1s1i",
	"/$s$s/5s5s/1s1l",
	"/$s$s/5s5s/1s1i/4s4a",
	"/$s$s/5s5s/1s1i/4s4h",
	"/$s$s/5s5s/1s1l/4s4a",
	"/$s$s/5s5s/1s1l/4s4h",
	"/$s$s/5s5s/4s4a",
	"/$s$s/5s5s/4s4h",
	"/$s$s/5s5s/4s4a",
	"/$s$s/5s5s/4s4h",
	"/$s$s/1s1i",
	"/$s$s/1s1l",
	"/$s$s/1s1i/4s4a",
	"/$s$s/1s1i/4s4h",
	"/$s$s/1s1l/4s4a",
	"/$s$s/1s1l/4s4h",
	"/$s$s/4s4a",
	"/$s$s/4s4h",
	"/$s$s/4s4a",
	"/$s$s/4s4h",
	"/0s0o",
	"/0s0o/2s2a",
	"/0s0o/2s2a/3s3e",
	"/0s0o/2s2a/3s3e/5s5s",
	"/0s0o/2s2a/3s3e/5s5s/1s1i",
	"/0s0o/2s2a/3s3e/5s5s/1s1l",
	"/0s0o/2s2a/3s3e/5s5s/1s1i/4s4a",
	"/0s0o/2s2a/3s3e/5s5s/1s1i/4s4h",
	"/0s0o/2s2a/3s3e/5s5s/1s1l/4s4a",
	"/0s0o/2s2a/3s3e/5s5s/1s1l/4s4h",
	"/0s0o/2s2a/3s3e/5s5s/4s4a",
	"/0s0o/2s2a/3s3e/5s5s/4s4h",
	"/0s0o/2s2a/3s3e/5s5s/4s4a",
	"/0s0o/2s2a/3s3e/5s5s/4s4h",
	"/0s0o/2s2a/3s3e/1s1i",
	"/0s0o/2s2a/3s3e/1s1l",
	"/0s0o/2s2a/3s3e/1s1i/4s4a",
	"/0s0o/2s2a/3s3e/1s1i/4s4h",
	"/0s0o/2s2a/3s3e/1s1l/4s4a",
	"/0s0o/2s2a/3s3e/1s1l/4s4h",
	"/0s0o/2s2a/3s3e/4s4a",
	"/0s0o/2s2a/3s3e/4s4h",
	"/0s0o/2s2a/3s3e/4s4a",
	"/0s0o/2s2a/3s3e/4s4h",
	"/0s0o/2s2a/5s5s",
	"/0s0o/2s2a/5s5s/1s1i",
	"/0s0o/2s2a/5s5s/1s1l",
	"/0s0o/2s2a/5s5s/1s1i/4s4a",
	"/0s0o/2s2a/5s5s/1s1i/4s4h",
	"/0s0o/2s2a/5s5s/1s1l/4s4a",
	"/0s0o/2s2a/5s5s/1s1l/4s4h",
	"/0s0o/2s2a/5s5s/4s4a",
	"/0s0o/2s2a/5s5s/4s4h",
	"/0s0o/2s2a/5s5s/4s4a",
	"/0s0o/2s2a/5s5s/4s4h",
	"/0s0o/2s2a/1s1i",
	"/0s0o/2s2a/1s1l",
	"/0s0o/2s2a/1s1i/4s4a",
	"/0s0o/2s2a/1s1i/4s4h",
	"/0s0o/2s2a/1s1l/4s4a",
	"/0s0o/2s2a/1s1l/4s4h",
	"/0s0o/2s2a/4s4a",
	"/0s0o/2s2a/4s4h",
	"/0s0o/2s2a/4s4a",
	"/0s0o/2s2a/4s4h",
	"/0s0o/3s3e",
	"/0s0o/3s3e/5s5s",
	"/0s0o/3s3e/5s5s/1s1i",
	"/0s0o/3s3e/5s5s/1s1l",
	"/0s0o/3s3e/5s5s/1s1i/4s4a",
	"/0s0o/3s3e/5s5s/1s1i/4s4h",
	"/0s0o/3s3e/5s5s/1s1l/4s4a",
	"/0s0o/3s3e/5s5s/1s1l/4s4h",
	"/0s0o/3s3e/5s5s/4s4a",
	"/0s0o/3s3e/5s5s/4s4h",
	"/0s0o/3s3e/5s5s/4s4a",
	"/0s0o/3s3e/5s5s/4s4h",
	"/0s0o/3s3e/1s1i",
	"/0s0o/3s3e/1s1l",
	"/0s0o/3s3e/1s1i/4s4a",
	"/0s0o/3s3e/1s1i/4s4h",
	"/0s0o/3s3e/1s1l/4s4a",
	"/0s0o/3s3e/1s1l/4s4h",
	"/0s0o/3s3e/4s4a",
	"/0s0o/3s3e/4s4h",
	"/0s0o/3s3e/4s4a",
	"/0s0o/3s3e/4s4h",
	"/0s0o/5s5s",
	"/0s0o/5s5s/1s1i",
	"/0s0o/5s5s/1s1l",
	"/0s0o/5s5s/1s1i/4s4a",
	"/0s0o/5s5s/1s1i/4s4h",
	"/0s0o/5s5s/1s1l/4s4a",
	"/0s0o/5s5s/1s1l/4s4h",
	"/0s0o/5s5s/4s4a",
	"/0s0o/5s5s/4s4h",
	"/0s0o/5s5s/4s4a",
	"/0s0o/5s5s/4s4h",
	"/0s0o/1s1i",
	"/0s0o/1s1l",
	"/0s0o/1s1i/4s4a",
	"/0s0o/1s1i/4s4h",
	"/0s0o/1s1l/4s4a",
	"/0s0o/1s1l/4s4h",
	"/0s0o/4s4a",
	"/0s0o/4s4h",
	"/0s0o/4s4a",
	"/0s0o/4s4h",
	"/2s2a",
	"/2s2a/3s3e",
	"/2s2a/3s3e/5s5s",
	"/2s2a/3s3e/5s5s/1s1i",
	"/2s2a/3s3e/5s5s/1s1l",
	"/2s2a/3s3e/5s5s/1s1i/4s4a",
	"/2s2a/3s3e/5s5s/1s1i/4s4h",
	"/2s2a/3s3e/5s5s/1s1l/4s4a",
	"/2s2a/3s3e/5s5s/1s1l/4s4h",
	"/2s2a/3s3e/5s5s/4s4a",
	"/2s2a/3s3e/5s5s/4s4h",
	"/2s2a/3s3e/5s5s/4s4a",
	"/2s2a/3s3e/5s5s/4s4h",
	"/2s2a/3s3e/1s1i",
	"/2s2a/3s3e/1s1l",
	"/2s2a/3s3e/1s1i/4s4a",
	"/2s2a/3s3e/1s1i/4s4h",
	"/2s2a/3s3e/1s1l/4s4a",
	"/2s2a/3s3e/1s1l/4s4h",
	"/2s2a/3s3e/4s4a",
	"/2s2a/3s3e/4s4h",
	"/2s2a/3s3e/4s4a",
	"/2s2a/3s3e/4s4h",
	"/2s2a/5s5s",
	"/2s2a/5s5s/1s1i",
	"/2s2a/5s5s/1s1l",
	"/2s2a/5s5s/1s1i/4s4a",
	"/2s2a/5s5s/1s1i/4s4h",
	"/2s2a/5s5s/1s1l/4s4a",
	"/2s2a/5s5s/1s1l/4s4h",
	"/2s2a/5s5s/4s4a",
	"/2s2a/5s5s/4s4h",
	"/2s2a/5s5s/4s4a",
	"/2s2a/5s5s/4s4h",
	"/2s2a/1s1i",
	"/2s2a/1s1l",
	"/2s2a/1s1i/4s4a",
	"/2s2a/1s1i/4s4h",
	"/2s2a/1s1l/4s4a",
	"/2s2a/1s1l/4s4h",
	"/2s2a/4s4a",
	"/2s2a/4s4h",
	"/2s2a/4s4a",
	"/2s2a/4s4h",
	"/3s3e",
	"/3s3e/5s5s",
	"/3s3e/5s5s/1s1i",
	"/3s3e/5s5s/1s1l",
	"/3s3e/5s5s/1s1i/4s4a",
	"/3s3e/5s5s/1s1i/4s4h",
	"/3s3e/5s5s/1s1l/4s4a",
	"/3s3e/5s5s/1s1l/4s4h",
	"/3s3e/5s5s/4s4a",
	"/3s3e/5s5s/4s4h",
	"/3s3e/5s5s/4s4a",
	"/3s3e/5s5s/4s4h",
	"/3s3e/1s1i",
	"/3s3e/1s1l",
	"/3s3e/1s1i/4s4a",
	"/3s3e/1s1i/4s4h",
	"/3s3e/1s1l/4s4a",
	"/3s3e/1s1l/4s4h",
	"/3s3e/4s4a",
	"/3s3e/4s4h",
	"/3s3e/4s4a",
	"/3s3e/4s4h",
	"/5s5s",
	"/5s5s/1s1i",
	"/5s5s/1s1l",
	"/5s5s/1s1i/4s4a",
	"/5s5s/1s1i/4s4h",
	"/5s5s/1s1l/4s4a",
	"/5s5s/1s1l/4s4h",
	"/5s5s/4s4a",
	"/5s5s/4s4h",
	"/5s5s/4s4a",
	"/5s5s/4s4h",
	"/1s1i",
	"/1s1l",
	"/1s1i/4s4a",
	"/1s1i/4s4h",
	"/1s1l/4s4a",
	"/1s1l/4s4h",
	"/4s4a",
	"/4s4h",
	"/4s4a",
	"/4s4h",
	/* done */
	NULL
};

/* Result of traversing the binary search tree. */
static int search_result;
/* Password is based on this word found in the dictionary. */
static char found_word[MAXWORDLEN];

static int
compare_passwords(const void *a, const void *b)
{
	const stored_password_t *pwd_a = (const stored_password_t *)a;
	const stored_password_t *pwd_b = (const stored_password_t *)b;

	return (strcmp(pwd_a->password, pwd_b->password));
}

/* ARGSUSED */
static void
walk_search_tree(const void *node, VISIT visit, int level)
{
	if ((search_result == 0) && ((visit == preorder) || (visit == leaf))) {
		stored_password_t *pwd = *(stored_password_t **)node;

		if (FindPW(pwd->pwp, pwd->password) != pwd->pwd_notfound) {
			search_result = pwd->type;
			(void) strlcpy(found_word, pwd->password,
			    sizeof (found_word));
		}
	}
}

/*
 * Stores a mangled password in the binary search tree,
 * if not already there.
 */
static int
store_password(const char *password, int type, void **search_tree,
    PWDICT *pwp, uint32_t pwd_notfound)
{
	size_t password_length = strlen(password);
	void *found;
	stored_password_t *pwd;

	/* All words in the dictionary are up to this limit long. */
	if (password_length >= MAXWORDLEN)
		password_length = MAXWORDLEN - 1;

	pwd = malloc(sizeof (stored_password_t) + password_length + 1);
	if (pwd == NULL)
		return (DICT_LOOKUP_ERR_MEMORY);

	pwd->type = type;
	pwd->pwp = pwp;
	pwd->pwd_notfound = pwd_notfound;
	(void) strlcpy(pwd->password, password, password_length + 1);

	found = tsearch(pwd, search_tree, compare_passwords);
	if (found == NULL) {
		free(pwd);
		return (DICT_LOOKUP_ERR_MEMORY);
	}
	if (*(stored_password_t **)found != pwd)
		free(pwd); /* mangled password already present */

	return (0);
}

/*
 * Mangles the password and stores all possibilities into the
 * search tree. The password buffer must have at least STRINGSIZE size.
 */
int
FascistLook(PWDICT *pwp, uint32_t notfound, char *password,
	void **search_tree)
{
	int i, ret;

	/*
	 * it should be safe to use Mangle with its reliance on PATH_SIZE
	 * since password cannot be longer than TRUNCSTRINGSIZE;
	 * nonetheless this is not an elegant solution
	 */

	for (i = 0; r_destructors[i]; i++) {
		char *a;

		if (!(a = Mangle(password, r_destructors[i]))) {
			continue;
		}

		ret = store_password(a, DICTIONARY_WORD, search_tree,
		    pwp, notfound);
		if (ret != 0)
			return (ret);
	}

	(void) strlcpy(password, Reverse(password), STRINGSIZE);

	for (i = 0; r_destructors[i]; i++) {
		char *a;

		if (!(a = Mangle(password, r_destructors[i]))) {
			continue;
		}

		ret = store_password(a, REVERSE_DICTIONARY_WORD,
		    search_tree, pwp, notfound);
		if (ret != 0)
			return (ret);
	}

	return (0);
}

int
DictCheck(char *password, char *path, char *dict_word, size_t dict_word_size)
{
	PWDICT *pwp;
	int r = 0;
	char substring[STRINGSIZE];
	char rpassword[STRINGSIZE];
	size_t length, offset;
	size_t passwd_length;
	uint32_t notfound;
	void *search_tree = NULL;

	if ((pwp = PWOpen(path, "rF")) == NULL)
		return (DATABASE_OPEN_FAIL);

	notfound = PW_WORDS(pwp);

	(void) strlcpy(rpassword, password, TRUNCSTRINGSIZE);
	(void) strcpy(rpassword, Lowercase(rpassword));
	(void) Trim(rpassword);
	passwd_length = strlen(rpassword);

	/*
	 * Iterates over all possible substring length and their starting
	 * offsets. There are exactly n * (n + 1) / 2
	 * (where n = password length) possibilities, so that's not too
	 * hard on the system. For example if password length is 15,
	 * then we try only 105 substrings. Not so bad for such infrequent
	 * operation such as changing the password.
	 */
	for (length = 1; length <= passwd_length; length++) {
		for (offset = 0; offset <= passwd_length - length; offset++) {
			(void) strlcpy(substring, rpassword + offset,
			    STRINGSIZE);
			substring[length] = '\0';

			r = FascistLook(pwp, notfound, substring, &search_tree);
			if (r != 0)
				goto exit;
		}
	}

	search_result = 0;
	found_word[0] = '\0';
	twalk(search_tree, walk_search_tree);
	r = search_result;
	if ((r == DICTIONARY_WORD) || (r == REVERSE_DICTIONARY_WORD))
		(void) strlcpy(dict_word, found_word, dict_word_size);

exit:
	(void) PWClose(pwp);
	tdestroy(search_tree, free);
	return (r);
}
