#!/usr/bin/python

#
# Copyright (c) 2017, 2020, Oracle and/or its affiliates.
#

#
# Check for any AK divergence keywords
#
# We currently look for three:
#       AKFORK          - used in mapfiles, C code, and as a general marker
#       FISH_BUILD      - used in Makefiles and package manifests
#       _SC_SUNW_FEATURE_AK - used in C code
#

import re
import sys

akwords = re.compile(r"AKFORK|FISH_BUILD|_SC_SUNW_FEATURE_AK")


def akfork(fh, filename=None, output=sys.stderr):
    """Search FH for any AK fork keywords, to warn during pbchk.

    If any are found, instruct the developer to check both variants."""

    if not filename:
        filename = fh.name

    for line in fh:
        line = line.rstrip("\r\n")

        match = akwords.findall(line)
        if match:
            output.write(
                "   %s contains AK divergence.\n" % filename)
            return 1

    return 0
