#
# Copyright (c) 2021, Oracle and/or its affiliates.
#

#
# Pkgfmt check.
# Catching pkgfmt errors in "hg nits" provides another early place where
# we can flag these before spending time in building the source, only to
# find something as trivial as a pkgfmt error caused a build abort.
#

import sys
from onbld.Checks.ProcessCheck import processcheck


def pkgfmtchk(fh, filename=None, output=sys.stderr, **opts):
    opttrans = {
        "check_only": "-c",
        "show_diff": "-d",
    }

    for x in [x for x in opts if x not in opttrans]:
        raise TypeError("pkgfmtchk() got an unexpected keyword argument %s" %
                        x)

    options = [opttrans[x] for x in opts if opts[x]]

    if not filename:
        filename = fh.name

    options.append(filename)

    ret, tmpfile = processcheck("pkgfmt", options, [], output)

    if tmpfile:
        if opts["show_diff"]:
            output.write(filename + ":\n")
        for line in tmpfile:
            # Trim the pkgfmt error message
            line = line.replace("; run pkgfmt on file without -c or -d to "
                                "reformat manifest in place", "")
            if not opts["show_diff"]:
                line = filename + ": " + line
            output.write(line)

        tmpfile.close()
    return ret
