#!/usr/bin/python

#
# Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
#

#
# The 'checks' package contains various checks that may be run
#

import os
import socket
import sys


#
# Generic check to test if a host is on OWAN
#
def onOWAN():
    try:
        if socket.gethostbyname("bug.oraclecorp.com"):
            return True
        else:
            return False
    except socket.herror:
        return False


#
# Generic grabber of user email address
#
try:
    import ldap
    # Force the immediate load of ldap
    ldap.initialize
except ImportError:
    ldap = None


def get_user_email():

    name = os.getenv("LOGNAME")
    if not name:
        import pwd
        name = pwd.getpwuid(os.getuid()).pw_name

    # Check to see if we're running on a gate machine as 'hg', in which case
    # we should use configuration specific to that gate.
    if name == "hg":
        try:
            sys.path.insert(
                0, os.path.realpath(
                    os.path.join(os.path.dirname(__file__), "..")))
            from etc import config
            if "@" in config.GATEKEEPER:
                return config.GATEKEEPER
            else:
                return "%s_%s@%s" % (
                    config.GATE_NAME, config.GATEKEEPER, config.GATE_MAIL)
        except ImportError:
            pass

    if not ldap:
        return None

    lo = ldap.initialize("ldap://ldap.oracle.com")
    lo.simple_bind_s()
    res = lo.search_s(
        "dc=oracle,dc=com", ldap.SCOPE_SUBTREE,
        "(&(objectClass=person)(uid=%s))" % name)
    lo.unbind_s()
    if not res:
        return None
    # There should be only one result.
    return res[0][1]["mail"][0]
