#
# Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
#

if [[ -z "$1" ]]; then
	print -u2 "Usage: compare_reci.ksh <workspace>"
	exit 2
fi

ORIG_REPO=$1
RECI_REPO=${1}-reci

cp -rP $ORIG_REPO $RECI_REPO 
cd $RECI_REPO

$HG reci -m "Test Squish" > /tmp/reci.$$.out || exit 250
grep -v 'Do you want to backup files first' /tmp/reci.$$.out
rm /tmp/reci.$$.out

gdiff -rux .hg $ORIG_REPO $RECI_REPO || exit 251

$HG -R $ORIG_REPO list > orig.out.$$
$HG -R $RECI_REPO list > squish.out.$$

$HG branch -R $ORIG_REPO > orig-branch.out.$$
$HG branch -R $RECI_REPO > squish-branch.out.$$

# As of mercurial 1.5, "hg heads" gives you the heads of all branches,
# whether or or not they're topological heads.  Determine whether or not we
# need to pass -t to "hg heads".
$HG heads -t -R $ORIG_REPO > /dev/null 2>&1 && TOPO_HEADS=-t

# Assume that the reci'd head is the parent of the original repo 
ORIG_HEAD=$($HG parent -q -R $ORIG_REPO )
$HG heads -q $TOPO_HEADS -R $ORIG_REPO | grep -v $ORIG_HEAD > orig-heads.out.$$

# ... and that the new changeset will be the tip of the reci'd repo
RECI_HEAD=$($HG tip -q -R $RECI_REPO)
$HG heads -q $TOPO_HEADS -R $RECI_REPO | grep -v $RECI_HEAD > squish-heads.out.$$

cmp orig.out.$$ squish.out.$$ || exit 252
cmp orig-branch.out.$$ squish-branch.out.$$ || exit 253
cmp orig-heads.out.$$ squish-heads.out.$$ || exit 254

(cd $ORIG_REPO && stat --printf="%n %a %A %F %N\n" *) > orig-fsstat.out.$$
(cd $RECI_REPO && stat --printf="%n %a %A %F %N\n" * | grep -v '\.out\.') > squish-fsstat.out.$$

cmp orig-fsstat.out.$$ squish-fsstat.out.$$ || exit 255
