/*
 * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 */

#ifndef _SYS_VERIFY_KEY_H
#define	_SYS_VERIFY_KEY_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * The Solaris elfsign RSA-2048 public key modulus is from:
 * openssl x509 -noout -modulus -in /etc/certs/elfsign/Oracle_Solaris_2017.pem
 * openssl x509 -noout -text    -in /etc/certs/elfsign/Oracle_Solaris_2017.pem
 *
 * The DN (subject) is the following (no trailing '\n' and NUL characters):
 *	O=Oracle Corporation, OU=Corporate Object Signing,
 *	OU=Solaris Signed Execution, CN=Oracle Solaris 2017
 *	(from openssl x509 -noout -subject -in Oracle_Solaris_2017.pem)
 */

static const uchar_t	default_dn_hash[VERIFY_DN_HASH_LEN] = {
    0xf9, 0xbb, 0x8b, 0xf1, 0xe7, 0x4d, 0x99, 0xf6,
    0x3d, 0x26, 0x5b, 0x36, 0x2d, 0x83, 0x8c, 0xcd,
    0xff, 0xd6, 0x6a, 0xaa, 0xdd, 0xbb, 0x1f, 0xf3,
    0xa2, 0x45, 0x83, 0xd9, 0xfb, 0xed, 0x0b, 0xf6 };

static const uchar_t	default_modulus[VERIFY_MODULUS_LEN] = {
    0xe8, 0x1f, 0x0c, 0xeb, 0x5c, 0x02, 0x9a, 0x97,
    0x59, 0xb3, 0xa2, 0xe6, 0xcd, 0xd3, 0x04, 0x1e,
    0x9c, 0xbf, 0x32, 0x9f, 0x04, 0xf3, 0x7c, 0xe4,
    0xbb, 0x8f, 0xf6, 0x95, 0x02, 0xcf, 0x93, 0xb6,
    0xdf, 0x4f, 0xc8, 0x19, 0x8d, 0xc8, 0xb4, 0xbe,
    0x67, 0x79, 0x1d, 0x58, 0xa2, 0x90, 0x7d, 0x37,
    0x35, 0xfb, 0x39, 0xa0, 0x46, 0x87, 0x6c, 0xf4,
    0x47, 0x71, 0xab, 0x78, 0xe0, 0x91, 0x2d, 0x3d,
    0x00, 0x9a, 0x44, 0x13, 0x01, 0x22, 0x36, 0xec,
    0x24, 0x53, 0x4c, 0xac, 0x23, 0x5d, 0xcf, 0x08,
    0xe7, 0xce, 0xf7, 0x0a, 0x99, 0x5e, 0x55, 0x83,
    0xd0, 0xaa, 0xe8, 0x54, 0x60, 0x23, 0x9e, 0xef,
    0xc1, 0x46, 0x65, 0x29, 0x61, 0xd1, 0xf7, 0x47,
    0x3f, 0xbe, 0x61, 0x42, 0x2b, 0x00, 0xad, 0xea,
    0xb5, 0xef, 0xf3, 0xe6, 0xc7, 0x74, 0xd6, 0xc9,
    0xe7, 0xe5, 0xc6, 0x70, 0xde, 0xd4, 0x41, 0x7c,
    0xcd, 0x53, 0x27, 0xb5, 0x12, 0x56, 0x38, 0xd5,
    0xd7, 0x95, 0xce, 0x7b, 0x9f, 0x66, 0xa0, 0x22,
    0x66, 0x2a, 0xc7, 0xf4, 0xcd, 0x97, 0xdc, 0x70,
    0x12, 0xe6, 0xac, 0x7c, 0x96, 0x46, 0xb9, 0xce,
    0xe9, 0x04, 0x43, 0x8b, 0x97, 0x3a, 0xc5, 0x19,
    0xed, 0x05, 0xc3, 0x91, 0xd8, 0x45, 0x3b, 0x70,
    0x4c, 0xea, 0xc7, 0x74, 0x29, 0x0d, 0x54, 0xaf,
    0x33, 0xff, 0x8c, 0x21, 0x5e, 0xd7, 0xf1, 0xd7,
    0xf8, 0x72, 0xc1, 0xf9, 0xb3, 0xad, 0x42, 0x1c,
    0xa2, 0x21, 0x3c, 0x5e, 0x88, 0x29, 0x97, 0xf0,
    0x5d, 0x30, 0xdc, 0x3e, 0x91, 0xe3, 0x12, 0xb0,
    0xc3, 0x1e, 0x17, 0x39, 0x59, 0xcc, 0x8b, 0x4a,
    0x69, 0xe5, 0xd6, 0xc7, 0x78, 0xcd, 0x2b, 0x87,
    0xff, 0x22, 0x52, 0x3e, 0x02, 0x3d, 0x66, 0x00,
    0xab, 0x46, 0x92, 0x9e, 0xaa, 0xb3, 0x6f, 0x28,
    0xba, 0x1e, 0x52, 0xf4, 0x7a, 0xae, 0x98, 0x75};

/*
 * The Solaris 12 elfsign RSA-2048 public key modulus is the same as the
 * Solaris 2017 public key modulus.
 *
 * The DN (subject) is the following (no trailing '\n' and NUL characters):
 *	O=Oracle Corporation, OU=Corporate Object Signing,
 *	OU=Solaris Signed Execution, CN=Solaris 12
 *	(from openssl x509 -noout -subject -in /etc/certs/elfsign/ORCLS12SE)
 */
static const uchar_t	s12_dn_hash[VERIFY_DN_HASH_LEN] = {
    0xd1, 0x3e, 0x21, 0xf5, 0x0a, 0xeb, 0x71, 0xb0,
    0xf9, 0x56, 0x28, 0xae, 0x9a, 0xc3, 0x4c, 0x92,
    0x19, 0x03, 0x15, 0x11, 0x8b, 0x83, 0xed, 0xce,
    0xd3, 0x45, 0xee, 0xf7, 0x42, 0x93, 0x0b, 0xfc };

#define	s12_modulus	default_modulus

/*
 * The Solaris 11 elfsign RSA-2048 public key modulus is from:
 *	openssl x509 -noout -modulus -in /etc/certs/elfsign/ORCLS11SE
 *	openssl x509 -noout -text    -in /etc/certs/elfsign/ORCLS11SE
 *
 * The DN (subject) is the following (no trailing '\n' and NUL characters):
 *	O=Oracle Corporation, OU=Corporate Object Signing,
 *	OU=Solaris Signed Execution, CN=Solaris 11
 *	(from openssl x509 -noout -subject -in /etc/certs/elfsign/ORCLS11SE)
 */

static const uchar_t	s11_dn_hash[VERIFY_DN_HASH_LEN] = {
    0x3a, 0xf4, 0xd6, 0x8d, 0x07, 0x8e, 0xa4, 0x79,
    0x45, 0x4f, 0xc8, 0xdc, 0xee, 0xa7, 0xa7, 0x1f,
    0xb4, 0x81, 0xaf, 0x71, 0x13, 0x6a, 0xff, 0x16,
    0x2a, 0xfb, 0x00, 0x21, 0x60, 0xc3, 0x5f, 0x2d };

static const uchar_t	s11_modulus[VERIFY_MODULUS_LEN] = {
    0xb5, 0xac, 0x83, 0x91, 0x56, 0x39, 0x0f, 0x3d,
    0x04, 0xfe, 0x82, 0x51, 0x5b, 0xdd, 0xe6, 0x80,
    0xad, 0x2b, 0x18, 0x64, 0x81, 0x25, 0x1c, 0xae,
    0xbe, 0xb7, 0x4b, 0xcd, 0x19, 0x91, 0x0e, 0xd9,
    0xde, 0xfc, 0xf7, 0x8a, 0x23, 0x21, 0x5a, 0xe3,
    0x46, 0x1f, 0x5f, 0x06, 0xa4, 0x3d, 0x81, 0xe1,
    0xe1, 0x06, 0xe1, 0xb0, 0x1f, 0xe0, 0xf7, 0xae,
    0xaf, 0xcf, 0xb5, 0xcd, 0x3d, 0x5d, 0x9f, 0x56,
    0x20, 0x94, 0x3b, 0x2d, 0xa2, 0xb6, 0x8a, 0x0a,
    0x31, 0xaa, 0xa0, 0x76, 0x04, 0x57, 0xa8, 0xa2,
    0x9c, 0x30, 0x3a, 0xa1, 0x08, 0x8f, 0xe0, 0x70,
    0x3b, 0x1b, 0x73, 0x95, 0x1c, 0x09, 0x01, 0xe9,
    0x03, 0x25, 0x3f, 0x59, 0x28, 0xfb, 0x4c, 0x89,
    0xe6, 0x5c, 0x35, 0x37, 0x42, 0x8e, 0x78, 0x9a,
    0x5f, 0x73, 0xa9, 0xee, 0x39, 0xd5, 0x0e, 0x3d,
    0x70, 0xd9, 0xf7, 0xaf, 0xa0, 0xea, 0xf0, 0xdb,
    0x1d, 0x58, 0xcf, 0x0e, 0x0a, 0x8f, 0x11, 0x64,
    0x6d, 0xc1, 0xcb, 0x84, 0x53, 0xe6, 0x10, 0x66,
    0xc1, 0xfa, 0x92, 0x4c, 0x8b, 0xa2, 0xd4, 0xae,
    0x10, 0xb0, 0x1c, 0xfd, 0x29, 0x3f, 0xfd, 0xef,
    0xb7, 0x80, 0x44, 0x85, 0x55, 0xd2, 0x3f, 0xbe,
    0xce, 0x1e, 0xd2, 0x15, 0xea, 0x79, 0x10, 0x38,
    0x43, 0xa4, 0xb0, 0x26, 0xa8, 0x24, 0xab, 0x7c,
    0xef, 0x49, 0xb0, 0x9f, 0xd9, 0xa1, 0xb0, 0x26,
    0xf1, 0x71, 0x6e, 0xce, 0x6e, 0x4d, 0x14, 0x08,
    0x59, 0x7c, 0x38, 0xb1, 0x23, 0xf0, 0xef, 0x1f,
    0x9c, 0x41, 0x8f, 0xe5, 0x34, 0x71, 0xdf, 0x07,
    0xaa, 0xb8, 0x6c, 0xcd, 0x8b, 0x23, 0x2c, 0x2d,
    0xd5, 0xc4, 0x67, 0x0c, 0x4f, 0xfd, 0x06, 0xbe,
    0x32, 0x6b, 0x3b, 0x1b, 0x6d, 0xb5, 0x84, 0xfa,
    0xb2, 0x3d, 0x7e, 0x68, 0x19, 0xe0, 0x83, 0xc9,
    0xb1, 0xca, 0xd7, 0x8a, 0x6c, 0x9b, 0xe0, 0x39 };

#ifdef	__cplusplus
}
#endif

#endif /* _SYS_VERIFY_KEY_H */
