/*
 * Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
 */

#ifndef _SYS_ZVNET_COMMON_H
#define	_SYS_ZVNET_COMMON_H

#include <sys/types.h>

#ifdef	__cplusplus
extern "C" {
#endif

#define	ZVNET_INVALID_ID	((uint16_t)-1)

/*
 * In general the Hypercalls are grouped into sets with a block comment for
 * each set. A host supports all or none of the calls in a set, which usually
 * implements a particular feature or a group of related features that go
 * together. In the event of a partial port of a set to another OS update,
 * the comments need to be updated on that source to indicate the new grouping
 * along with relevant code changes. Hypercall numbers and signatures cannot
 * change since they constitute an ABI between the guest and the host.
 *
 * All hypercalls up to ZVNET_SET_PROMISC are guaranteed on any host that
 * supports kernel zones and represent the very first implementation.
 */
#define	ZVNET_FEATURE		0	/* Check feature. */
#define	ZVNET_GET_MAC_ADDR	1	/* Get MAC address. */
#define	ZVNET_GET_MAXSDU	2	/* Get max SDU. */
#define	ZVNET_GET_CKSUM		3	/* Get checksum offload capability. */
#define	ZVNET_GET_MAC_ADDRS 	4	/* Get all MAC addresses. */
#define	ZVNET_ADD_MCAST		5	/* Add a multicast address. */
#define	ZVNET_DEL_MCAST		6	/* Remove a multicast address. */
#define	ZVNET_REG_DESC_V0	7	/* Register descriptor pages. */
#define	ZVNET_TX		8	/* Transmit a packet. */
#define	ZVNET_TX_SMALL		9	/* Transmit a small packet. */
#define	ZVNET_INTR_ON		10	/* Enable interrupts. */
#define	ZVNET_INTR_OFF		11	/* Disable interrupts. */
#define	ZVNET_GET_LSO		12	/* Get LSO capability */
#define	ZVNET_SET_PROMISC	13	/* Set promiscuous state. */

/*
 * Next set of hypercalls that allow multiple Tx/Rx rings. These are part of
 * the V1 ring version.
 */
#define	ZVNET_TX_RING		14	/* Transmit a packet on a ring. */
#define	ZVNET_TX_SMALL_RING	15	/* Transmit a small packet on a ring. */
#define	ZVNET_REG_DESC_V1	16	/* Register descriptor pages. */

/*
 * Next set of hypercalls that implement LRO (V1) support.
 */
#define	ZVNET_GET_LRO		17	/* Get LRO capability */
#define	ZVNET_GET_LRO_BUFSIZE	18	/* Get LRO buffer size */
					/* for capable NIC */
#define	ZVNET_SET_LRO_DISP	19	/* Set LRO disposition of zvnet host */

/*
 * Next set of additional hypercalls for LRO V2 support.
 */
#define	ZVNET_INTR_NOTIFY	20	/* Enable notification interrupt */
#define	ZVNET_GET_LRO_V2	21	/* Get current LRO status */

/*
 * Next set of hypercalls as noted in the side comments.
 */
#define	ZVNET_GET_IFSPEED	22	/* Get link speed of the zvnet host */
#define	ZVNET_GET_LINK_STATE	23	/* Get link state of the zvnet host */

/*
 * Hypercalls to support IPoIB link. Ended up retrofitting multicast and macaddr
 * hypercalls to a new V2 version.
 */
#define	ZVNET_GET_MEDIA_TYPE	24	/* Get media type of the zvnet host */
#define	ZVNET_GET_MAC_ADDR_V2	25	/* Get MAC address. */
#define	ZVNET_ADD_MCAST_V2	26	/* Add a multicast address. */
#define	ZVNET_DEL_MCAST_V2	27	/* Remove a multicast address. */
#define	ZVNET_GET_MCAST_SDU	28	/* Get multicast SDU. Neded for IPoIB */
#define	ZVNET_IPOIB_LINK	29	/* Use IPoIB link */

/*
 * Hypercall to allow guest to provide true failover capability
 */
#define	ZVNET_GET_PHYSLINK_STATE 30	/* Get the physical link state of the */
					/* lower link of the zvnet host */
/*
 * Hypercall to allow LSO for IPv6
 */
#define	ZVNET_GET_LSO_V2	31	/* GET LSO capability version 2 */

/*
 * Hypercall to allow VLAN support for KZ.
 */
#define	ZVNET_GET_VLAN_IDS	32 	/* Get zvnet vlan ids */

/*
 * Next set of hypercalls to allow dynamic creation of VLANs by KZ
 */
#define	ZVNET_ADD_DYNAMIC_VLAN	33	/* Add VLAN ID dynamically */
#define	ZVNET_REM_DYNAMIC_VLAN	34	/* Remove VLAN ID dynamically */
#define	ZVNET_ADD_DYNAMIC_MACADDR 35	/* Add MAC address dynamically */
#define	ZVNET_REM_DYNAMIC_MACADDR 36	/* Remove MAC address dynamically */

/*
 * Hypercall to get the pkey of the IPoIB interface on the host.
 */
#define	ZVNET_GET_PKEY		37	/* Get pkey of the zvnet host */

/*
 * Next set of ring version V2 related calls for better performance.
 */
#define	ZVNET_REG_DESC_V2	38	/* Ring registration version 2 */
#define	ZVNET_TX_RING_WORKER	39

/*
 * Hypercall to get the IOV config
 */
#define	ZVNET_GET_IOV_CONF	40	/* Get iov of the zvnet host */
#define	ZVNET_CHK_RANDOM_MACADDR 41    /* Check Random MAC address  */
/* Change this and add code to zks.c if new hypercalls are added */
#define	ZKS_NET_LAST		ZVNET_CHK_RANDOM_MACADDR

#define	ZVNET_TX_RING_MAX_V1	7
#define	ZVNET_RX_RING_MAX_V1	7

#define	ZVNET_TX_RING_DEF_V2	8
#define	ZVNET_RX_RING_DEF_V2	8

#define	ZVNET_TX_RING_DEF_40G	16
#define	ZVNET_RX_RING_DEF_40G	16

/*
 * Total of 33 interrupts  max limits us to at most 16 rings for Tx and Rx
 * with 1 interrupt per ring, plus 1 intr for ZVNET_INTERRUPT_NOTIFY
 */
#define	ZVNET_TX_RING_MAX_V2	16
#define	ZVNET_RX_RING_MAX_V2	16

/* Should be the maximum of all the ring versions */
#define	ZVNET_TX_RING_MAX	16
#define	ZVNET_RX_RING_MAX	16

/*
 * Required features. Once a guest uses them the host must support them on
 * resume or the resume should fail:
 *     ZVNET_IPOIB_LINK
 */
#define	ZVNET_REQ_COUNT		(1)

/*
 * For performance reasons, align receive buffers on a 32 bit boundary
 * to avoid pullupmsg() at every layer above the driver that inspects
 * the layer 3 and higher headers.
 */
#define	ETHER_IPHDR_ALIGN_ROOM	2

/*
 * Interrupt numbers listed below. New interrupts can be added, but the numbers
 * of existing interrupts can't change due to guest host compatibility
 * requirements. The current mapping scheme can handle up to 16 Rx and 16 Tx
 * interrupts plus 1 for ZVNET_NOTIFY_INTERRUPT
 */
#define	ZVNET_INTERRUPT_RX	0
#define	ZVNET_INTERRUPT_TX	1

#define	ZVNET_RX_INTERRUPT_BASE_V1	2
#define	ZVNET_TX_INTERRUPT_BASE_V1					\
	(ZVNET_RX_INTERRUPT_BASE_V1 + ZVNET_RX_RING_MAX_V1)

#define	ZVNET_INTERRUPT_NOTIFY	16

#define	ZVNET_RX_INTERRUPT_BASE_V2	0
#define	ZVNET_TX_INTERRUPT_BASE_V2	(ZVNET_INTERRUPT_NOTIFY + 1)

#define	ZVNET_TX_RING2INTR(ver, ring)					\
	((ver) == ZVNET_RING_V2 ? ZVNET_TX_INTERRUPT_BASE_V2 + (ring) :	\
	((ver) == ZVNET_RING_V1 ? ZVNET_TX_INTERRUPT_BASE_V1 + (ring) :	\
	ZVNET_INTERRUPT_TX))

#define	ZVNET_RX_RING2INTR(ver, ring)					\
	((ver) == ZVNET_RING_V2 ? ZVNET_RX_INTERRUPT_BASE_V2 + (ring) :	\
	((ver) == ZVNET_RING_V1 ? ZVNET_RX_INTERRUPT_BASE_V1 + (ring) :	\
	ZVNET_INTERRUPT_RX))

#define	ZVNET_RING_COUNT_ASSERT(zvnp)					\
	ASSERT(((zvnp)->zg_ring_version == ZVNET_RING_V2 && 		\
	(zvnp)->zg_rx_ring_count <= ZVNET_RX_RING_MAX_V2 &&		\
	(zvnp)->zg_tx_ring_count <= ZVNET_TX_RING_MAX_V2) ||		\
	((zvnp)->zg_ring_version == ZVNET_RING_V1 && 			\
	(zvnp)->zg_rx_ring_count <= ZVNET_RX_RING_MAX_V1 &&		\
	(zvnp)->zg_tx_ring_count <= ZVNET_TX_RING_MAX_V1) ||		\
	((zvnp)->zg_ring_version == ZVNET_RING_V0 && 			\
	(zvnp)->zg_rx_ring_count == 1 && (zvnp)->zg_tx_ring_count == 1))

/*
 * Ring version numbers
 */
#define	ZVNET_RING_UNKNOWN	-1
#define	ZVNET_RING_V0		0
#define	ZVNET_RING_V1		1
#define	ZVNET_RING_V2		2
#define	ZVNET_LATEST_RING_VERSION	ZVNET_RING_V2

/*
 * Link speeds.
 */
#define	ZVNET_1G	(1000 * 1000 * 1000UL)
#define	ZVNET_10G	(10 * ZVNET_1G)
#define	ZVNET_40G	(40 * ZVNET_1G)

#define	ZVNET_16K	(16 * 1024)

/*
 * The various shared ring or data pages are registered by the guest
 * as a block.
 *
 * All hosts support v0.
 */
typedef struct zvnet_reg_v0 {
	uint64_t tx_pa;		/* TX descriptor page. */
	uint64_t tx_n;
	uint64_t rx_pa;		/* RX descriptor page. */
	uint64_t rx_n;
	uint64_t tx_small;	/* TX small packet page. */
	uint64_t tx_complete;	/* TX completion page. */
	uint64_t rx_complete;	/* RX completion page. */
	uint64_t rx_push;	/* RX push page. */
} zvnet_reg_v0_t;

/*
 * Support for the ZVNET_TX_RING hypercall implies support for the v1
 * registration call.
 */
typedef struct zvnet_reg_v1 {
	uint64_t tx_offset;	/* TX rings offset. */
	uint64_t tx_count;	/* TX ring count. */

	uint64_t rx_offset;	/* RX rings offset. */
	uint64_t rx_count;	/* RX ring count. */

	/*
	 * At tx_offset bytes are quadruples of uint64_t that specify
	 * a set of TX rings. In each quadruples:
	 *
	 * - the GPA of the completion ring page,
	 * - the GPA of the descriptor page,
	 * - the number of descriptors,
	 * - the GPA of the small packet page.
	 *
	 * At rx_offset bytes are quadruples of uint64_t that specify
	 * a set of RX rings. In each quadruple:
	 *
	 * - the GPA of the completion ring page,
	 * - the GPA of the push ring page,
	 * - the GPA of the descriptor page,
	 * - the number of descriptors.
	 */
} zvnet_reg_v1_t;

/*
 * *********************************************************************
 * Ring V0 and V1 descriptor structures below
 * *********************************************************************
 *
 * The transmit descriptor page contains an array of
 * zvnet_tx_desc_t. They are referenced by index.
 *
 * A chain through `next' describes a single packet.
 */
typedef	struct zvnet_tx_desc {
	uint64_t guest_pa;	/* Guest PA of this segment. */
	uint64_t guest_cookie;	/* Guest identifier for the request. */
	uint16_t length;	/* Of this segment (<= PAGESIZE). */
	uint16_t next;		/* Index of next segment in packet. */
	uint32_t cksum_start;
	uint32_t cksum_stuff;
	uint32_t cksum_end;
	uint32_t cksum_value;
	uint32_t cksum_flags;
	uint32_t lso_flags;
	uint32_t lso_mss;
} zvnet_tx_desc_t;

/*
 * The receive descriptor page contains an array of
 * zvnet_rx_desc_t. They are referenced by index.
 */
typedef struct zvnet_rx_desc {
	uint64_t guest_pa;	/* Buffer is guest physically contiguous. */
	/*
	 * The length field neeeds to be increased to a 32 bit field to
	 * accommodate the IPoIB CM MTU of 64KB. We have length_ld (lower digit)
	 * and length_ud (upper digit). length_ld and upper_ld stores lower and
	 * upper 16 bits of the length value. Old kernel will use only
	 * length_ld, which will be mapped to its length field. Therefore, we
	 * guarantee backward compatibility.
	 */
	uint16_t length_ld;
	uint16_t length_ud;
	uint32_t cksum_start;
	uint32_t cksum_stuff;
	uint32_t cksum_end;
	uint32_t cksum_value;
	uint32_t cksum_flags;
} zvnet_rx_desc_t;

/*
 * A packet for transmission may consist of up to TX_MAX_SEG distinct
 * {guest_pa, len} segments.
 *
 * We would like to support a 64k MTU and allow for two segments per
 * page.
 */
#define	TX_MAX_SEG	(2 * (((64 * 1024) / PAGESIZE) + 1))

#define	ZVNET_PORT_VLANID	0x1

/* Structure for passing VLAN id info. */
typedef struct zvnet_vid_info_s {
	uint16_t zvi_vid;
	uint16_t zvi_flags;
} zvnet_vid_info_t;

/* We cannot use MAXMACADDRLEN as we break the ABI if it changes */
#define	ZVNET_MAXMACADDRLEN	20

/*
 * Structure for passing dynamic MAC address info in
 * ZVNET_ADD_DYNAMIC_MACADDR and ZVNET_REM_DYNAMIC_MACADDR hypercalls
 */
typedef struct zvnet_dynamic_macaddr {
	uint8_t zdm_addr[ZVNET_MAXMACADDRLEN];
	size_t zdm_addr_len;
} zvnet_dynamic_macaddr_t;

typedef enum {
	ZVNET_IOV_PREF_AUTO,
	ZVNET_IOV_PREF_ON,
	ZVNET_IOV_PREF_OFF
} zvnet_iov_pref_t;

/*
 * *********************************************************************
 * Ring V2 descriptor and registration structures below
 * *********************************************************************
 */

#define	ZVNET_RX_DESC_MIN_COUNT_V2	32
#define	ZVNET_RX_DESC_MAX_COUNT_V2	16384

#define	ZVNET_TX_DESC_MIN_COUNT_V2	64
#define	ZVNET_TX_DESC_MAX_COUNT_V2	32768

#define	ZVNET_TX_SP_MIN_COUNT_V2	32
#define	ZVNET_TX_SP_MAX_COUNT_V2	16384

#define	ZVNET_TX_SP_MIN_SIZE_V2		32
#define	ZVNET_TX_SP_MAX_SIZE_V2		512

#define	ZVNET_TX_CP_DESC_MIN_COUNT_V2	32
#define	ZVNET_TX_CP_DESC_MAX_COUNT_V2	16384

/*
 *
 * Shared ring head and tail pointers. Fillers to ensure each pointer
 * is in a different cache line.
 */
typedef struct zvnets_ring_htptrs_s {
	uint16_t	zsrp_head;
	uint16_t	zsrp_filler1[62];
	uint16_t	zsrp_tail;
	uint16_t	zsrp_filler2[62];
	uint16_t	zsrp_clast;
	uint16_t	zsrp_filler3[62];
} zvnets_ring_htptrs_t;

#define	ZVNET_HTPTRS_PAGES_MAX						\
	(P2ROUNDUP((ZVNET_TX_RING_MAX_V2 + ZVNET_RX_RING_MAX_V2) *	\
	sizeof (zvnets_ring_htptrs_t), PAGESIZE) >> PAGESHIFT)

typedef struct zvnet_reg_v2 {
	uint8_t  zrv2_version;			/* ZVNET_RING_V2 */
	uint8_t	 zrv2_pad1;
	uint16_t zrv2_pad2;
	uint16_t zrv2_tx_ring_count;   		/* TX ring count. */
	uint16_t zrv2_rx_ring_count;   		/* RX ring count. */

	uint16_t zrv2_tx_desc_count;		/* Tx desc count per ring */
	uint16_t zrv2_tx_cp_desc_count;		/* Tx cp desc count per ring */
	uint16_t zrv2_rx_desc_count;		/* Rx desc count per ring */
	uint16_t zrv2_tx_small_pkt_count; 	/* Small pkt cnt per Tx ring */
	uint16_t zrv2_tx_small_pkt_size;  	/* size of small pkt buffer */

	uint64_t zrv2_ring_htptrs_gpa;		/* Shared ring pointers GPA */
	uint64_t zrv2_ring_htptrs_bufsize;	/* pointer area buffer size */
	uint64_t zrv2_tx_ring_gpa[ZVNET_TX_RING_MAX];
	uint64_t zrv2_tx_small_gpa[ZVNET_TX_RING_MAX];
	uint64_t zrv2_rx_ring_gpa[ZVNET_RX_RING_MAX];
} zvnet_reg_v2_t;

/*
 * The shared transmit descriptor area between the guest and host contains an
 * array of Tx descriptors. Each Tx descriptor is 16 bytes long and fixed in
 * size. This design permits interpreting any 16 byte aligned entity in the
 * shared area as a Tx descriptor. Tx descriptors are designed to be small,
 * fit exactly in cache lines, encourage sequential access of descriptor memory
 * and minimize pointer indirections and list manipulations.
 *
 * The 1st byte indicates the type of Tx descriptor. The 1st Tx descriptor
 * for a packet is the context descriptor. This is followed by the guest
 * cookie descriptor and 1 or more segment descriptors. A Tx packet may have
 * an inter-mingled set of small and large segment descriptors. However a Tx
 * packet must not exceed a total of ZSTD_MAX_SEG descriptors.
 */
/* ctxd_type */
#define	ZSTD_CTX_DESC			1
#define	ZSTD_GUEST_COOKIE_DESC		2
#define	ZSTD_LARGE_SEG_DESC		3
#define	ZSTD_SMALL_SEG_DESC		4

/* ctxd_flags */
#define	ZSTD_TX_READY			1

#define	ZSTD_MAX_SEG			32	/* Max Tx desc. per packet */
#define	TXP_MAX_HDL			16	/* Must be <= ZSTD_MAX_SEG */

/*
 * 1 context descriptor + 1 guest cookie descriptor overhead for every
 * Tx packet
 */
#define	TX_DESC_OVERHEAD 2

/* Context Descriptor describing LSO and checksum related information */
typedef struct zvnets_tx_ctx_desc_s {
	uint8_t
		ctxd_type : 4,
		ctxd_flags : 4;
	uint8_t	ctxd_nsegs;		/* Number of segments in this pkt */
	uint16_t ctxd_lso_flags;
	uint16_t ctxd_lso_mss;
	uint16_t ctxd_cksum_flags;

	uint16_t ctxd_cksum_start;
	uint16_t ctxd_cksum_stuff;
	uint16_t ctxd_cksum_end;
	uint16_t ctxd_cksum_value;
} zvnets_tx_ctx_desc_t;

/*
 * Transmit completion notification to the guest passes back the guest
 * cookie to the guest to help in identification of the completion.
 */
typedef struct zvnets_tx_gcookie_desc_s {
	uint8_t
			gcd_type : 4,	/* ZSTD_GUEST_COOKIE_DESC */
			gcd_pad : 4;
	uint8_t		gcd_pad1;
	uint16_t	gcd_pad2;
	uint32_t	gcd_pad3;
	uint64_t	gcd_cookie;	/* guest cookie supplied by guest */
} zvnets_tx_gcookie_desc_t;

#define	LSD_PREMAPPED 1
/*
 * Each segment descriptor identifies a memory segment of the Tx packet
 * Large segment descriptors pass the guest physical address (GPA) of
 * packet segments which will be mapped by the host to host based I/O address
 * for actual DMA
 */
typedef struct zvnets_tx_large_seg_desc_s {
	uint8_t
			lsd_type : 4,	/* ZSTD_LARGE_SEG_DESC */
			lsd_pad : 4;
	uint8_t		lsd_flags;
	uint16_t	lsd_pad2;
	uint32_t	lsd_len;	/* Length of the segment */
	uint64_t	lsd_gpa;	/* Guest physical address of the seg */
} zvnets_tx_large_seg_desc_t;

/*
 * Small packet segments are copied by the guest into a shared small
 * packet segment buffer area and the index of the small segment is passed
 * in the descriptor
 */
typedef struct zvnets_tx_small_seg_desc_s {
	uint8_t
			ssd_type : 4,	/* ZSTD_SMALL_SEG_DESC */
			ssd_pad: 4;
	uint8_t		ssd_pad1;
	uint16_t	ssd_index;	/* Index of the small buffer */
	uint32_t	ssd_len;	/* Actual length */
	uint64_t	ssd_pad2;
} zvnets_tx_small_seg_desc_t;

/* Generic Tx descriptor type */
typedef struct zvnets_tx_desc_s {
	uint8_t		txd_type;
	uint8_t		txd_pad1[7];
	uint64_t	txd_pad2;
} zvnets_tx_desc_t;

/*
 * The receive descriptor pages contain an array of zvnets_rx_desc_t shared
 * between the guest and host. Each descriptor is 32 bytes and describes a
 * single Rx packet. The packet buffer memory is allocated in the guest using
 * one of numaio_contig_alloc(), contig_mem_alloc() or ddi_dma_mem_alloc()
 * This ensures that a single Rx packet buffer of at most 64K size is a
 * single virtually and physically contiguous chunk of memory in both guest
 * and host.
 *
 * There is an exception case where Rx packets may be chained using multiple
 * descriptors with 1 descriptor per PAGESIZEd buffer. This exception happens
 * in the event of memory allocation failure during Rx setup when the MTU
 * is > PAGESIZE.
 */
typedef struct zvnets_rx_desc_s {
	uint64_t zsrd_gpa;	/* Buffer is guest and host phys contiguous. */

	uint32_t zsrd_length;	/* Of this buffer/packet max 64 KB */
	uint16_t zsrd_flags;
	uint16_t zsrd_ndesc;	/* Number of descriptors used for the packet */

	uint16_t zsrd_offset;	/* Media offset */
	uint16_t zsrd_cksum_start;
	uint16_t zsrd_cksum_stuff;
	uint16_t zsrd_cksum_end;

	uint16_t zsrd_cksum_value;
	uint16_t zsrd_cksum_flags;
	uint16_t zsrd_db_hash;
	uint16_t zsrd_pad1;
} zvnets_rx_desc_t;

/* zsrd_flags */
#define	ZSRD_DB_HASH	0x1

/*
 * Macros to operate on producer-consumer type circular rings with a head/tail
 * pointer. All parameters passed to these macros must be uint16_t.
 * Uses unsigned arithmetic on uint16_t.
 */
#define	RING_EMPTY(head, tail)	((head) == (tail))

#define	RING_ELEM_COUNT(head, tail, ring_size)				\
	((tail) < (head) ?						\
	(tail) + (ring_size) - (head) :					\
	(tail) - (head))

/*
 * To differentiate between full and empty rings, we need to leave the
 * last slot of a ring empty.
 */
#define	RING_FULL(head, tail, ring_size)				\
	(RING_ELEM_COUNT(head, tail, ring_size) == (ring_size) - 1)

#define	RING_ELEM_NEXT(index, ring_size)				\
	(((index) + 1) % (ring_size))

#define	RING_ELEM_PREV(index, ring_size)				\
	(((index) + (ring_size) - 1) % (ring_size))

#define	RING_ELEM_SKIP(index, skipcnt, ring_size)			\
	(((index) + (skipcnt)) % (ring_size))

/*
 * Macro to check that element 'mid' is in between 'head' and 'tail' in
 * the circular ring, but excluding 'head' and 'tail'.
 */
#define	RING_ELEM_BETWEEN(mid, head, tail, ring_size)			\
	((tail) < (head) ?						\
	!((mid) >= (tail) && (mid) <= (head)) :				\
	(mid) > (head) && (mid) < (tail))

/* Same as above, but inclusive of 'head' and 'tail' */
#define	RING_ELEM_BETWEEN_INCL(mid, head, tail, ring_size)		\
	((tail) < (head) ?						\
	!((mid) > (tail) && (mid) < (head)) :				\
	(mid) >= (head) && (mid) <= (tail))

/* Does the ring have space to take 'nelem' more elements ? */
#define	RING_HAS_SPACE(head, tail, ring_size, nelem)			\
	(RING_ELEM_COUNT(head, tail, ring_size) + (nelem) < (ring_size))

#ifdef	__cplusplus
}
#endif

#endif /* _SYS_ZVNET_COMMON_H */
