/*
 * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 */

#ifndef	_SYS_BOOTINFO_H
#define	_SYS_BOOTINFO_H

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * The 32 bit kernel loader code needs to build several structures
 * that the kernel is expecting. They will contain native sized
 * pointers for the 64 bit kernel.
 */

struct boot_memlist {
	uint64_t	addr;
	uint64_t	size;
	uint64_t	next;
	uint64_t	prev;
};

/*
 * The kernel needs to know how to find its modules.
 */
struct boot_modules {
	uint64_t	bm_addr;
	uint32_t	bm_size;
	uint32_t	bm_padding;
};

/*
 * For Kernel Zones: xboot_info is passed by zvboot and members must not be
 * added or removed between bi_next_paddr and bi_framebuffer_info, since zvboot
 * is a standalone program that may be out of synch with unix (dboot doesn't
 * have this problem).
 *
 */
#pragma pack(1)
struct xboot_info {
	uint64_t	bi_next_paddr;	/* next physical address not used */
	uint64_t	bi_next_vaddr;	/* next virtual address not used */
	uint64_t	bi_cmdline;
	uint64_t	bi_phys_install;
	uint64_t	bi_rsvdmem;
	uint64_t	bi_xboot_version;	/* xboot interface version */
	uint64_t	bi_pcimem;
	uint64_t	bi_modules;
	uint32_t	bi_module_cnt;
	uint32_t	bi_use_largepage;	/* MMU uses large pages */
	uint32_t	bi_use_pae;	/* MMU uses PAE mode (8 byte PTES) */
	uint32_t	bi_use_nx;	/* MMU uses NX bit in PTEs */
	uint32_t	bi_use_pge;	/* MMU uses Page Global Enable */
	uint64_t	bi_pt_window;
	uint64_t	bi_pte_to_pt_window;
	uint64_t	bi_kseg_size;	/* size used for kernel nucleus pages */
	uint64_t	bi_top_page_table;
	uint64_t	bi_mb_info;	/* multiboot 1 or 2 info structure */
	int		bi_mb_version;	/* Multiboot version (for bi_mb_info) */
	uint64_t	bi_acpi_rsdp;	/* Pointer to the ACPI RSDP in the */
					/* Multiboot2 Info structure -- this */
					/* MUST be copied out before boot is */
					/* released (same goes for SMBIOS, */
					/* below). */
	uint32_t	bi_acpi_rsdp_len; /* Length of rsdp^^ structure */
	uint64_t	bi_uefi_systab;	/* Pointer to UEFI System Table */
	int		bi_uefi_arch;	/* See below */
	uint64_t	bi_smbios_entry; /* SMBIOS Entry Point struct address */
					/* address points in the multiboot2 */
					/* info structure */
	uint32_t	bi_smbios_entry_len;  /* Buffer length needed for ^^ */
	uint32_t	bi_vesa_mode;	/* XBI_VBE_MODE_INVALID means */
					/* NO MODE INFORMATION */
	/*
	 * Whether we're supplied multiboot v1 or v2 information, the
	 * framebuffer information from either structure is placed into
	 * a [multiboot v2-defined] multiboot_tag_framebuffer structure
	 * to normalize the information and eliminate superfluous code
	 * in consumers of framebuffer information.
	 */
	uint64_t	bi_framebuffer_info; /* multiboot_tag_framebuffer ptr */

	uint64_t	bi_boot_alloc;	/* lists for early boot allocator */
	uint64_t	bi_boot_valloc;	/* lists for early boot allocator */
	uint64_t	bi_bootalloc_next_addr;		/* paddr */
	uint64_t	bi_bootvalloc_next_addr;	/* vaddr */
	int		bi_uefi_usebsmem;
};
#pragma pack()

#define	XBI_VERSION_1	1	/* initial version; advanced boot allocator */
#define	XBI_VERSION_2	2	/* added bi_uefi_usebsmem to xboot_info */
#define	XBI_VERSION	XBI_VERSION_2

#define	XBI_VBE_MODE_INVALID	(~(uint32_t)0)

/*
 * For bi_uefi_arch
 */
#define	XBI_UEFI_ARCH_UNKNOWN	0
#define	XBI_UEFI_ARCH32		1
#define	XBI_UEFI_ARCH64		2

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_BOOTINFO_H */
